﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;
using Infragistics.Controls.Schedules;
using System.IO;
using Infragistics.Controls.Grids;
using Infragistics.Controls.Schedules.Primitives;
using System.Windows.Controls.Primitives;

namespace GanttSLIssue
{
    public partial class MainPage : UserControl
    {
        ScrollBar sb;

        public MainPage()
        {
            InitializeComponent();
        }

        private void saveBtn_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.Filter = "MS Project XML (.xml)|*.xml|All Files (*.*)|*.*";

            bool? isSaved = dialog.ShowDialog();
            if (isSaved == true)
            {
                try
                {
                    using (Stream stream = dialog.OpenFile())
                    {
                        SaveProjectToStream(stream);
                        stream.Close();
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }

        private void SaveProjectToStream(Stream stream)
        {
    // obtain the xamGantt Project
            var project = this.xamGantt.Project;
            project.SaveAsProjectXml(stream);
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {

           sb.Value++;
               
         
        }

        private void VerticalScrollBar_Loaded(object sender, RoutedEventArgs e)
        {
            sb = (sender as ScrollBar);
        }

   
    }

    public class ProjectViewModel : INotifyPropertyChanged
    {
        #region Private variables
        private ObservableCollection<ProjectView> views;
        private ObservableCollection<ProjectTable> tables;
        private ObservableCollection<ProjectColumn> columns;
        private Project project;
        #endregion // Private variables
        #region Public Properties
        public Project Project
        {
            get
            {
                if (project == null)
                {
                    project = ProjectDataHelper.GenerateProjectData();
                }
                return project;
            }
            set
            {
                if (value != project)
                {
                    project = value;
                    OnPropertyChanged("Project");
                }
            }
        }
        public ObservableCollection<ProjectColumn> Columns
        {
            get
            {
                if (columns == null)
                {
                    columns = GenerateColumns();
                }
                return columns;
            }
        }
        public ObservableCollection<ProjectTable> Tables
        {
            get
            {
                if (tables == null)
                {
                    tables = GenerateTables();
                }
                return tables;
            }
        }
        public ObservableCollection<ProjectView> Views
        {
            get
            {
                if (views == null)
                {
                    views = GenerateViews();
                }
                return views;
            }
        }
        #endregion // Public Properties
        #region Private helper methods
        private ObservableCollection<ProjectView> GenerateViews()
        {
            ObservableCollection<ProjectView> views = new ObservableCollection<ProjectView>();
            ProjectView view = new ProjectView
            {
                Key = "Standard01",
                TableKey = "Table 01",
                Settings = new ProjectViewSettings
                {
                    AreSummaryTasksVisible = true,
                    AreCriticalTasksHighlighted = true,
                    IsOutlineStructurePreservedWhenSorting = true,
                    NonWorkingTimeHighlightStyle = NonWorkingTimeHighlightStyle.ActualNonWorkingHours
                }
            };
            view.SortedColumns.Add(new ProjectColumnSortDescription { ColumnKey = "ManualStart" });
            view.SortedColumns.Add(new ProjectColumnSortDescription { ColumnKey = "ManualFinish", Direction = ListSortDirection.Descending });
            views.Add(view);
            view = new ProjectView { Key = "Standard02", TableKey = "Table 02" };
            views.Add(view);
            return views;
        }
        private ObservableCollection<ProjectTable> GenerateTables()
        {
            var columnsMap = Columns.ToDictionary(c => c.Key);
            ObservableCollection<ProjectTable> tables = new ObservableCollection<ProjectTable>();
            ProjectTable table = new ProjectTable
            {
                Key = "Table 01",

                ShowInMenu = true
            };
            table.Columns.Add(columnsMap["TaskName"]);
            table.Columns.Add(columnsMap["ManualStart"]);
            table.Columns.Add(columnsMap["ManualDuration"]);
            table.Columns.Add(columnsMap["ManualFinish"]);
            tables.Add(table);
            table = new ProjectTable { Key = "Table 02", ShowInMenu = true };
            table.Columns.Add(columnsMap["TaskName"]);
            table.Columns.Add(columnsMap["ManualStart"]);
            table.Columns.Add(columnsMap["ManualFinish"]);
            table.Columns.Add(columnsMap["Notes"]);
            tables.Add(table);
            return tables;
        }
        private ObservableCollection<ProjectColumn> GenerateColumns()
        {
            return new ObservableCollection<ProjectColumn>()
            {
                new ProjectColumn { Key = "TaskName", HeaderText = "Name"},
                new ProjectColumn { Key = "ManualStart", HeaderText = "Start", Settings = new ProjectColumnSettings { HeaderTextHorizontalAlignment = HorizontalAlignment.Center}},
                new ProjectColumn { Key = "ManualFinish", HeaderText = "Finish", Settings = new ProjectColumnSettings { HeaderTextHorizontalAlignment = HorizontalAlignment.Center}},
                new ProjectColumn { Key = "ManualDuration", HeaderText = "Duration"},
                new ProjectColumn { Key = "Notes", HeaderText = "Notes"}
            };
        }
        #endregion // Private helper methods
        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;
        public void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion // INotifyPropertyChanged
    }

    public class ProjectDataHelper
    {
        public static Project GenerateProjectData()
        {
            // Create a Project instance
            Project project = new Project();

            // Create a project root ProjectTask
            ProjectTask rootTask = new ProjectTask
            {
                TaskName = "Project Summary Task",
                IsManual = false
            };

            // Add the root task to the project
            project.RootTask.Tasks.Add(rootTask);

            // Dates in xamGantt are stored in UTC
            DateTime startTime = DateTime.Today.ToUniversalTime();

            // Add ProjectTask subtasks to the root project task
            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Planning",
                IsManual = false,
                Start = startTime,
                Duration = TimeSpan.FromHours(8),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Documenting",
                IsManual = false,
                Start = startTime,
                // Configure task duration
                ManualDuration = ProjectDuration.FromFormatUnits(2, ProjectDurationFormat.Days),
                // Set a task constraint
                ConstraintDate = DateTime.Today.AddDays(3).ToUniversalTime(),
                ConstraintType = ProjectTaskConstraintType.FinishNoLaterThan,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Creating Demo Application",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(3, ProjectDurationFormat.Days),
                ConstraintType = ProjectTaskConstraintType.MustStartOn,
                ConstraintDate = DateTime.Today.AddDays(2).ToUniversalTime(),
                IsEstimated = true,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Collecting Feedback",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(1, ProjectDurationFormat.Days),
                IsEstimated = true,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Designing",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(5, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Development",
                IsManual = true,
                // Manual tasks may have free text for Start and Finish date
                ManualStart = new ManualDateTime("Still In Discussion"),
                ManualDuration = ProjectDuration.FromFormatUnits(10, ProjectDurationFormat.Days),
                ManualFinish = new ManualDateTime("Not Defined Yet"),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Testing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(7, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Bug Fixing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(4, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Project Complete",
                IsManual = false,
                Start = startTime,
                // Set a milestone
                IsMilestone = true
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Planning",
                IsManual = false,
                Start = startTime,
                Duration = TimeSpan.FromHours(8),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Documenting",
                IsManual = false,
                Start = startTime,
                // Configure task duration
                ManualDuration = ProjectDuration.FromFormatUnits(2, ProjectDurationFormat.Days),
                // Set a task constraint
                ConstraintDate = DateTime.Today.AddDays(3).ToUniversalTime(),
                ConstraintType = ProjectTaskConstraintType.FinishNoLaterThan,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Creating Demo Application",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(3, ProjectDurationFormat.Days),
                ConstraintType = ProjectTaskConstraintType.MustStartOn,
                ConstraintDate = DateTime.Today.AddDays(2).ToUniversalTime(),
                IsEstimated = true,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Collecting Feedback",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(1, ProjectDurationFormat.Days),
                IsEstimated = true,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Designing",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(5, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Development",
                IsManual = true,
                // Manual tasks may have free text for Start and Finish date
                ManualStart = new ManualDateTime("Still In Discussion"),
                ManualDuration = ProjectDuration.FromFormatUnits(10, ProjectDurationFormat.Days),
                ManualFinish = new ManualDateTime("Not Defined Yet"),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Testing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(7, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Bug Fixing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(4, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Project Complete",
                IsManual = false,
                Start = startTime,
                // Set a milestone
                IsMilestone = true
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Planning",
                IsManual = false,
                Start = startTime,
                Duration = TimeSpan.FromHours(8),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Documenting",
                IsManual = false,
                Start = startTime,
                // Configure task duration
                ManualDuration = ProjectDuration.FromFormatUnits(2, ProjectDurationFormat.Days),
                // Set a task constraint
                ConstraintDate = DateTime.Today.AddDays(3).ToUniversalTime(),
                ConstraintType = ProjectTaskConstraintType.FinishNoLaterThan,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Creating Demo Application",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(3, ProjectDurationFormat.Days),
                ConstraintType = ProjectTaskConstraintType.MustStartOn,
                ConstraintDate = DateTime.Today.AddDays(2).ToUniversalTime(),
                IsEstimated = true,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Collecting Feedback",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(1, ProjectDurationFormat.Days),
                IsEstimated = true,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Designing",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(5, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Development",
                IsManual = true,
                // Manual tasks may have free text for Start and Finish date
                ManualStart = new ManualDateTime("Still In Discussion"),
                ManualDuration = ProjectDuration.FromFormatUnits(10, ProjectDurationFormat.Days),
                ManualFinish = new ManualDateTime("Not Defined Yet"),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Testing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(7, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Bug Fixing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(4, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Project Complete",
                IsManual = false,
                Start = startTime,
                // Set a milestone
                IsMilestone = true
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Planning",
                IsManual = false,
                Start = startTime,
                Duration = TimeSpan.FromHours(8),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Documenting",
                IsManual = false,
                Start = startTime,
                // Configure task duration
                ManualDuration = ProjectDuration.FromFormatUnits(2, ProjectDurationFormat.Days),
                // Set a task constraint
                ConstraintDate = DateTime.Today.AddDays(3).ToUniversalTime(),
                ConstraintType = ProjectTaskConstraintType.FinishNoLaterThan,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Creating Demo Application",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(3, ProjectDurationFormat.Days),
                ConstraintType = ProjectTaskConstraintType.MustStartOn,
                ConstraintDate = DateTime.Today.AddDays(2).ToUniversalTime(),
                IsEstimated = true,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Collecting Feedback",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(1, ProjectDurationFormat.Days),
                IsEstimated = true,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Designing",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(5, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Development",
                IsManual = true,
                // Manual tasks may have free text for Start and Finish date
                ManualStart = new ManualDateTime("Still In Discussion"),
                ManualDuration = ProjectDuration.FromFormatUnits(10, ProjectDurationFormat.Days),
                ManualFinish = new ManualDateTime("Not Defined Yet"),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Testing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(7, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Bug Fixing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(4, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Project Complete",
                IsManual = false,
                Start = startTime,
                // Set a milestone
                IsMilestone = true
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Planning",
                IsManual = false,
                Start = startTime,
                Duration = TimeSpan.FromHours(8),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Documenting",
                IsManual = false,
                Start = startTime,
                // Configure task duration
                ManualDuration = ProjectDuration.FromFormatUnits(2, ProjectDurationFormat.Days),
                // Set a task constraint
                ConstraintDate = DateTime.Today.AddDays(3).ToUniversalTime(),
                ConstraintType = ProjectTaskConstraintType.FinishNoLaterThan,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Creating Demo Application",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(3, ProjectDurationFormat.Days),
                ConstraintType = ProjectTaskConstraintType.MustStartOn,
                ConstraintDate = DateTime.Today.AddDays(2).ToUniversalTime(),
                IsEstimated = true,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Collecting Feedback",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(1, ProjectDurationFormat.Days),
                IsEstimated = true,
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Designing",
                IsManual = false,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(5, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Development",
                IsManual = true,
                // Manual tasks may have free text for Start and Finish date
                ManualStart = new ManualDateTime("Still In Discussion"),
                ManualDuration = ProjectDuration.FromFormatUnits(10, ProjectDurationFormat.Days),
                ManualFinish = new ManualDateTime("Not Defined Yet"),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Testing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(7, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Bug Fixing",
                IsManual = true,
                Start = startTime,
                ManualDuration = ProjectDuration.FromFormatUnits(4, ProjectDurationFormat.Days),
            });

            rootTask.Tasks.Add(new ProjectTask
            {
                TaskName = "Project Complete",
                IsManual = false,
                Start = startTime,
                // Set a milestone
                IsMilestone = true
            });

            return project;
        }
    }

 

}
